@@ -342,7 +342,7 @@ sendpacket_open_netmap(const char *device, char *errbuf, void *arg) {
 
     if (ioctl (sp->handle.fd, NIOCREGIF, &nmr)) {
         snprintf(errbuf, SENDPACKET_ERRBUF_SIZE, "Failure accessing netmap.\n"
-                "\tRequest for netmap version %u failed.\n\tCompiled netmap driver is version %u.\n\tError=%s\n",
+                "\tRequest for netmap version %d failed.\n\tCompiled netmap driver is version %d.\n\tError=%s\n",
                 sp->netmap_version, NETMAP_API, strerror(errno));
         goto NETMAP_IF_FAILED;
     }
@@ -502,9 +502,16 @@ void sendpacket_close_netmap(void *p)
     fprintf(stderr, "Switching network driver for %s to normal mode... ",
             sp->device);
     fflush(NULL);
-      /* flush any remaining packets */
+
+    /* flush any remaining packets */
     ioctl(sp->handle.fd, NIOCTXSYNC, NULL);
 
+    /* wait for traffic to be sent */
+    dbgx(2, "Waiting %d seconds for phy reset...", sp->netmap_delay);
+    sleep(sp->netmap_delay);
+    dbg(2, "Ready!");
+
+
 #ifdef linux
     if (!sp->is_vale) {
         /* restore original settings:
@@ -541,13 +548,16 @@ bool netmap_tx_queues_empty(void *p)
 
     assert(sp);
 
+    sp->cur_tx_ring = 0;
     txring = NETMAP_TXRING(sp->nm_if, sp->cur_tx_ring);
     while (NETMAP_TX_RING_EMPTY(txring)) {
         /* current ring is empty- go to next */
         ++sp->cur_tx_ring;
-        if (sp->cur_tx_ring > sp->last_tx_ring)
+        if (sp->cur_tx_ring > sp->last_tx_ring) {
             /* last ring */
+            sp->cur_tx_ring = 0;
             return true;
+        }
 
         txring = NETMAP_TXRING(sp->nm_if, sp->cur_tx_ring);
     }
