@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2017 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2018 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -239,7 +239,7 @@ sendpacket(sendpacket_t *sp, const u_char *data, size_t len, struct pcap_pkthdr
     assert(sp);
     assert(data);
 
-    if (len <= 0)
+    if (len == 0)
         return -1;
 
 TRY_SEND_AGAIN:
@@ -320,7 +320,7 @@ TRY_SEND_AGAIN:
                         break;
 
                     default:
-                        sendpacket_seterr(sp, "Error with %s [" COUNTER_SPEC "]: %s (errno = %d)", 
+                        sendpacket_seterr(sp, "Error with %s [" COUNTER_SPEC "]: %s (errno = %d)",
                                 INJECT_METHOD, sp->sent + sp->failed + 1, strerror(errno), errno);
                 }
             }
@@ -349,7 +349,7 @@ TRY_SEND_AGAIN:
                         break;
 
                     default:
-                        sendpacket_seterr(sp, "Error with %s [" COUNTER_SPEC "]: %s (errno = %d)", 
+                        sendpacket_seterr(sp, "Error with %s [" COUNTER_SPEC "]: %s (errno = %d)",
                                 INJECT_METHOD, sp->sent + sp->failed + 1, strerror(errno), errno);
                 }
             }
@@ -376,7 +376,7 @@ TRY_SEND_AGAIN:
                         break;
 
                     default:
-                        sendpacket_seterr(sp, "Error with %s [" COUNTER_SPEC "]: %s (errno = %d)", 
+                        sendpacket_seterr(sp, "Error with %s [" COUNTER_SPEC "]: %s (errno = %d)",
                                 INJECT_METHOD, sp->sent + sp->failed + 1, strerror(errno), errno);
                 }
             }
@@ -409,7 +409,7 @@ TRY_SEND_AGAIN:
                         break;
 
                     default:
-                        sendpacket_seterr(sp, "Error with %s [" COUNTER_SPEC "]: %s (errno = %d)", 
+                        sendpacket_seterr(sp, "Error with %s [" COUNTER_SPEC "]: %s (errno = %d)",
                                 INJECT_METHOD, sp->sent + sp->failed + 1, pcap_geterr(sp->handle.pcap), errno);
                 }
             }
@@ -436,6 +436,10 @@ TRY_SEND_AGAIN:
                 /* this indicates that a retry was requested - this is not a failure */
                 sp->retry_eagain ++;
                 retcode = 0;
+#ifdef HAVE_SCHED_H
+                /* yield the CPU so other apps remain responsive */
+                sched_yield();
+#endif
                 goto TRY_SEND_AGAIN;
             }
 #endif /* HAVE_NETMAP */
@@ -447,6 +451,8 @@ TRY_SEND_AGAIN:
 
     if (retcode < 0) {
         sp->failed ++;
+    } else if (sp->abort) {
+        sendpacket_seterr(sp, "User abort");
     } else if (retcode != (int)len) {
         sendpacket_seterr(sp, "Only able to write %d bytes out of %u bytes total",
                 retcode, len);
@@ -486,7 +492,6 @@ sendpacket_open(const char *device, char *errbuf, tcpr_dir_t direction,
                   errx(-1, "\"%s\" is a block device and is not a valid Tcpreplay device",
                       device);
                   break;
-                  break;
               case S_IFDIR:
                   errx(-1, "\"%s\" is a directory and is not a valid Tcpreplay device",
                       device);
@@ -867,9 +872,12 @@ sendpacket_open_pf(const char *device, char *errbuf)
     sendpacket_t *sp;
     struct ifreq ifr;
     struct sockaddr_ll sa;
-    int n = 1, err;
+    int err;
     socklen_t errlen = sizeof(err);
     unsigned int UNUSED(mtu) = 1500;
+#ifdef SO_BROADCAST
+    int n = 1;
+#endif
 
     assert(device);
     assert(errbuf);
@@ -888,7 +896,6 @@ sendpacket_open_pf(const char *device, char *errbuf)
         return NULL;
     }
 
-
     /* get the interface id for the device */
     if ((sa.sll_ifindex = get_iface_index(mysocket, device, errbuf)) < 0) {
         close(mysocket);
@@ -932,7 +939,7 @@ sendpacket_open_pf(const char *device, char *errbuf)
 
     /* make sure it's not loopback (PF_PACKET doesn't support it) */
     if (ifr.ifr_hwaddr.sa_family != ARPHRD_ETHER)
-        warnx("Unsupported physical layer type 0x%04x on %s.  Maybe it works, maybe it wont."
+        warnx("Unsupported physical layer type 0x%04x on %s.  Maybe it works, maybe it won't."
               "  See tickets #123/318", ifr.ifr_hwaddr.sa_family, device);
 
 #ifdef SO_BROADCAST
