@@ -3,7 +3,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2017 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2018 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -65,13 +65,12 @@ extern tcpedit_t *tcpedit;
 extern int debug;
 #endif
 
-static bool calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_time,
+static void calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_time,
         struct timeval *last, COUNTER len,
         sendpacket_t *sp, COUNTER counter, timestamp_t *sent_timestamp,
-        COUNTER *start_us, COUNTER *skip_length);
+        COUNTER start_us, COUNTER *skip_length);
 static void tcpr_sleep(tcpreplay_t *ctx, sendpacket_t *sp _U_,
-        struct timespec *nap_this_time, struct timeval *now,
-        tcpreplay_accurate accurate);
+        struct timespec *nap_this_time, struct timeval *now);
 static u_char *get_next_packet(tcpreplay_t *ctx, pcap_t *pcap,
         struct pcap_pkthdr *pkthdr,
         int file_idx,
@@ -227,7 +226,6 @@ fast_edit_packet_dl(struct pcap_pkthdr *pkthdr, u_char **pktdata,
     dbgx(1, "(%u): final src_ip=0x%08x dst_ip=0x%08x", iteration, src_ip, dst_ip);
 }
 
-#if defined HAVE_NETMAP
 static inline void wake_send_queues(sendpacket_t *sp, tcpreplay_opt_t *options)
 {
 #ifdef HAVE_NETMAP
@@ -235,7 +233,6 @@ static inline void wake_send_queues(sendpacket_t *sp, tcpreplay_opt_t *options)
         ioctl(sp->handle.fd, NIOCTXSYNC, NULL);   /* flush TX buffer */
 #endif
 }
-#endif /* HAVE_NETMAP */
 
 static inline void
 fast_edit_packet(struct pcap_pkthdr *pkthdr, u_char **pktdata,
@@ -475,8 +472,9 @@ void
 send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
 {
 
-    struct timeval print_delta, now, first_pkt_ts, pkt_ts_delta;
+    struct timeval print_delta, now, last_pkt_ts;
     tcpreplay_opt_t *options = ctx->options;
+    tcpreplay_stats_t *stats = &ctx->stats;
     COUNTER packetnum = 0;
     COUNTER limit_send = options->limit_send;
     struct pcap_pkthdr pkthdr;
@@ -490,19 +488,27 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
 #endif
     int datalink = options->file_cache[idx].dlt;
     COUNTER skip_length = 0;
-    COUNTER start_us;
     COUNTER end_us;
     bool preload = options->file_cache[idx].cached;
     bool top_speed = (options->speed.mode == speed_topspeed ||
             (options->speed.mode == speed_mbpsrate && options->speed.speed == 0));
-    bool now_is_now;
+    bool now_is_now = true;
+
+    gettimeofday(&now, NULL);
+    if (!timerisset(&stats->start_time)) {
+        TIMEVAL_SET(&stats->start_time, &now);
+        if (ctx->options->stats >= 0) {
+            char buf[64];
+            if (format_date_time(&stats->start_time, buf, sizeof(buf)) > 0)
+                printf("Test start: %s ...\n", buf);
+        }
+    }
 
     ctx->skip_packets = 0;
-    start_us = TIMEVAL_TO_MICROSEC(&ctx->stats.start_time);
-    timerclear(&first_pkt_ts);
-
+    timerclear(&last_pkt_ts);
     if (options->limit_time > 0)
-        end_us = start_us + SEC_TO_MICROSEC(options->limit_time);
+        end_us = TIMEVAL_TO_MICROSEC(&stats->start_time) +
+            SEC_TO_MICROSEC(options->limit_time);
     else
         end_us = 0;
 
@@ -512,13 +518,6 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
         prev_packet = NULL;
     }
 
-    if (!top_speed) {
-        gettimeofday(&now, NULL);
-        now_is_now = true;
-    } else {
-        now_is_now = false;
-    }
-
     /* MAIN LOOP 
      * Keep sending while we have packets or until
      * we've sent enough packets
@@ -541,8 +540,7 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
 
         /* Dual nic processing */
         if (ctx->intf2 != NULL) {
-
-            sp = (sendpacket_t *) cache_mode(ctx, options->cachedata, packetnum);
+            sp = (sendpacket_t *)cache_mode(ctx, options->cachedata, packetnum);
 
             /* sometimes we should not send the packet */
             if (sp == TCPR_DIR_NOSEND)
@@ -557,12 +555,6 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
         pktlen = options->use_pkthdr_len ? (COUNTER)pkthdr_ptr->len : (COUNTER)pkthdr_ptr->caplen;
 #endif
 
-        /* do we need to print the packet via tcpdump? */
-#ifdef ENABLE_VERBOSE
-        if (options->verbose)
-            tcpdump_print(options->tcpdump, &pkthdr, pktdata);
-#endif
-
         if (ctx->options->unique_ip && ctx->unique_iteration &&
                 ctx->unique_iteration > ctx->last_unique_iteration) {
             /* edit packet to ensure every pass has unique IP addresses */
@@ -575,12 +567,6 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
             update_flow_stats(ctx,
                     options->cache_packets ? sp : NULL, &pkthdr, pktdata, datalink);
 
-        if (ctx->first_time) {
-            /* get time and timestamp of the first packet */
-            gettimeofday(&now, NULL);
-            memcpy(&first_pkt_ts, &pkthdr.ts, sizeof(struct timeval));
-        }
-
         /*
          * this accelerator improves performance by avoiding expensive
          * time stamps during periods where we have fallen behind in our
@@ -595,86 +581,106 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
              * time stamping is expensive, but now is the
              * time to do it.
              */
-            dbgx(4, "This packet time: " TIMEVAL_FORMAT, pkthdr.ts.tv_sec, pkthdr.ts.tv_usec);
+            dbgx(4, "This packet time: " TIMEVAL_FORMAT, pkthdr.ts.tv_sec,
+                    pkthdr.ts.tv_usec);
             skip_length = 0;
             ctx->skip_packets = 0;
 
-            /*
-             * Only sleep if we're not in top speed mode (-t)
-             * or if the current packet is not late.
-             *
-             * This also sets skip_length which will avoid timestamping for
-             * a given number of packets.
-             */
-            timersub(&pkthdr.ts, &first_pkt_ts, &pkt_ts_delta);
-            if (calc_sleep_time(ctx, &pkt_ts_delta, &ctx->stats.last_time, pktlen, sp, packetnum,
-                    &ctx->stats.end_time, &start_us, &skip_length)) {
+            if (options->speed.mode == speed_multiplier) {
+                if (!timerisset(&last_pkt_ts)) {
+                    TIMEVAL_SET(&last_pkt_ts, &pkthdr.ts);
+                } else if (timercmp(&pkthdr.ts, &last_pkt_ts, >)) {
+                    struct timeval delta;
+
+                    timersub(&pkthdr.ts, &last_pkt_ts, &delta);
+                    timeradd(&stats->pkt_ts_delta, &delta, &stats->pkt_ts_delta);
+                    TIMEVAL_SET(&last_pkt_ts, &pkthdr.ts);
+                }
+
+                if (!timerisset(&stats->time_delta))
+                    TIMEVAL_SET(&stats->pkt_ts_delta, &stats->pkt_ts_delta);
+            }
+
+            if (!top_speed) {
                 now_is_now = true;
                 gettimeofday(&now, NULL);
             }
 
             /*
-             * track the time of the "last packet sent".  Again, because of OpenBSD
-             * we have to do a memcpy rather then assignment.
+             * Only if the current packet is not late.
+             *
+             * This also sets skip_length and skip_packets which will avoid
+             * timestamping for a given number of packets.
+             */
+            calc_sleep_time(ctx, &stats->pkt_ts_delta, &stats->time_delta,
+                    pktlen, sp, packetnum, &stats->end_time,
+                    TIMEVAL_TO_MICROSEC(&stats->start_time), &skip_length);
+
+            /*
+             * Track the time of the "last packet sent".
              *
              * A number of 3rd party tools generate bad timestamps which go backwards
              * in time.  Hence, don't update the "last" unless pkthdr.ts > last
              */
-            if (!top_speed && timercmp(&ctx->stats.last_time, &pkt_ts_delta, <)) 
-                memcpy(&ctx->stats.last_time, &pkt_ts_delta, sizeof(struct timeval));
+            if (timercmp(&stats->time_delta, &stats->pkt_ts_delta, <))
+                TIMEVAL_SET(&stats->time_delta, &stats->pkt_ts_delta);
 
             /*
              * we know how long to sleep between sends, now do it.
              */
-            if (timesisset(&ctx->nap))
-                tcpr_sleep(ctx, sp, &ctx->nap, &now, options->accurate);
+            if (!top_speed)
+                tcpr_sleep(ctx, sp, &ctx->nap, &now);
         }
 
-        dbgx(2, "Sending packet #" COUNTER_SPEC, packetnum);
-
-        /* write packet out on network */
-        if (sendpacket(sp, pktdata, pktlen, &pkthdr) < (int)pktlen)
-            warnx("Unable to send packet: %s", sendpacket_geterr(sp));
-
-        /*
-         * mark the time when we sent the last packet
-         *
-         * we have to cast the ts, since OpenBSD sucks
-         * had to be special and use bpf_timeval.
-         */
-        memcpy(&ctx->stats.end_time, &now, sizeof(ctx->stats.end_time));
-
-#ifdef TIMESTAMP_TRACE
-        add_timestamp_trace_entry(pktlen, &ctx->stats.end_time, skip_length);
+#ifdef ENABLE_VERBOSE
+        /* do we need to print the packet via tcpdump? */
+        if (options->verbose)
+            tcpdump_print(options->tcpdump, &pkthdr, pktdata);
 #endif
 
-        ctx->stats.pkts_sent++;
-        ctx->stats.bytes_sent += pktlen;
+        dbgx(2, "Sending packet #" COUNTER_SPEC, packetnum);
+        /* write packet out on network */
+        if (sendpacket(sp, pktdata, pktlen, &pkthdr) < (int)pktlen) {
+            warnx("Unable to send packet: %s", sendpacket_geterr(sp));
+            break;
+        }
+
+        /*
+         * Mark the time when we sent the last packet
+         */
+        TIMEVAL_SET(&stats->end_time, &now);
+
+#ifdef TIMESTAMP_TRACE
+        add_timestamp_trace_entry(pktlen, &stats->end_time, skip_length);
+#endif
+
+        stats->pkts_sent++;
+        stats->bytes_sent += pktlen;
 
         /* print stats during the run? */
         if (options->stats > 0) {
-            if (! timerisset(&ctx->stats.last_print)) {
-                memcpy(&ctx->stats.last_print, &now, sizeof(ctx->stats.last_print));
+            if (! timerisset(&stats->last_print)) {
+                TIMEVAL_SET(&stats->last_print, &now);
             } else {
-                timersub(&now, &ctx->stats.last_print, &print_delta);
+                timersub(&now, &stats->last_print, &print_delta);
                 if (print_delta.tv_sec >= options->stats) {
-                    memcpy(&ctx->stats.end_time, &now, sizeof(ctx->stats.end_time));
-                    packet_stats(&ctx->stats);
-                    memcpy(&ctx->stats.last_print, &now, sizeof(ctx->stats.last_print));
+                    TIMEVAL_SET(&stats->end_time, &now);
+                    packet_stats(stats);
+                    TIMEVAL_SET(&stats->last_print, &now);
                 }
             }
         }
 
 #if defined HAVE_NETMAP
-        if (sp->first_packet) {
+        if (sp->first_packet || timesisset(&ctx->nap)) {
             wake_send_queues(sp, options);
             sp->first_packet = false;
         }
 #endif
         /* stop sending based on the duration limit... */
-        if ((end_us > 0 && TIMEVAL_TO_MICROSEC(&now) > end_us) ||
+        if ((end_us > 0 && (COUNTER)TIMEVAL_TO_MICROSEC(&now) > end_us) ||
                 /* ... or stop sending based on the limit -L? */
-                (limit_send > 0 && ctx->stats.pkts_sent >= limit_send)) {
+                (limit_send > 0 && stats->pkts_sent >= limit_send)) {
             ctx->abort = true;
         }
     } /* while */
@@ -684,13 +690,13 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
     /* when completing test, wait until the last packet is sent */
     if (options->netmap && (ctx->abort || options->loop == 1)) {
         while (ctx->intf1 && !netmap_tx_queues_empty(ctx->intf1)) {
-            gettimeofday(&now, NULL);
             now_is_now = true;
+            gettimeofday(&now, NULL);
         }
 
         while (ctx->intf2 && !netmap_tx_queues_empty(ctx->intf2)) {
-            gettimeofday(&now, NULL);
             now_is_now = true;
+            gettimeofday(&now, NULL);
         }
     }
 #endif /* HAVE_NETMAP */
@@ -698,7 +704,7 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
     if (!now_is_now)
         gettimeofday(&now, NULL);
 
-    memcpy(&ctx->stats.end_time, &now, sizeof(ctx->stats.end_time));
+    TIMEVAL_SET(&stats->end_time, &now);
 
     increment_iteration(ctx);
 }
@@ -710,8 +716,9 @@ send_packets(tcpreplay_t *ctx, pcap_t *pcap, int idx)
 void
 send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *pcap2, int cache_file_idx2)
 {
-    struct timeval print_delta, now, first_pkt_ts, pkt_ts_delta;
+    struct timeval print_delta, now, last_pkt_ts;
     tcpreplay_opt_t *options = ctx->options;
+    tcpreplay_stats_t *stats = &ctx->stats;
     COUNTER packetnum = 0;
     COUNTER limit_send = options->limit_send;
     int cache_file_idx;
@@ -723,19 +730,27 @@ send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *
     packet_cache_t **prev_packet1 = NULL, **prev_packet2 = NULL;
     struct pcap_pkthdr *pkthdr_ptr;
     int datalink;
-    COUNTER start_us;
     COUNTER end_us;
     COUNTER skip_length = 0;
     bool top_speed = (options->speed.mode == speed_topspeed ||
             (options->speed.mode == speed_mbpsrate && options->speed.speed == 0));
-    bool now_is_now;
+    bool now_is_now = true;
+
+    gettimeofday(&now, NULL);
+    if (!timerisset(&stats->start_time)) {
+        TIMEVAL_SET(&stats->start_time, &now);
+        if (ctx->options->stats >= 0) {
+            char buf[64];
+            if (format_date_time(&stats->start_time, buf, sizeof(buf)) > 0)
+                printf("Dual test start: %s ...\n", buf);
+        }
+    }
 
     ctx->skip_packets = 0;
-    start_us = TIMEVAL_TO_MICROSEC(&ctx->stats.start_time);
-    timerclear(&first_pkt_ts);
-
+    timerclear(&last_pkt_ts);
     if (options->limit_time > 0)
-        end_us = start_us + SEC_TO_MICROSEC(options->limit_time);
+        end_us = TIMEVAL_TO_MICROSEC(&stats->start_time) +
+            SEC_TO_MICROSEC(options->limit_time);
     else
         end_us = 0;
 
@@ -747,17 +762,9 @@ send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *
         prev_packet2 = NULL;
     }
 
-
     pktdata1 = get_next_packet(ctx, pcap1, &pkthdr1, cache_file_idx1, prev_packet1);
     pktdata2 = get_next_packet(ctx, pcap2, &pkthdr2, cache_file_idx2, prev_packet2);
 
-    if (!top_speed) {
-        gettimeofday(&now, NULL);
-        now_is_now = true;
-    } else {
-        now_is_now = false;
-    }
-
     /* MAIN LOOP 
      * Keep sending while we have packets or until
      * we've sent enough packets
@@ -815,7 +822,6 @@ send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *
 
         dbgx(2, "packet " COUNTER_SPEC " caplen " COUNTER_SPEC, packetnum, pktlen);
 
-
 #if defined TCPREPLAY && defined TCPREPLAY_EDIT
         if (tcpedit_packet(tcpedit, &pkthdr_ptr, &pktdata, sp->cache_dir) == -1) {
             errx(-1, "Error editing packet #" COUNTER_SPEC ": %s", packetnum, tcpedit_geterr(tcpedit));
@@ -823,12 +829,6 @@ send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *
         pktlen = options->use_pkthdr_len ? (COUNTER)pkthdr_ptr->len : (COUNTER)pkthdr_ptr->caplen;
 #endif
 
-        /* do we need to print the packet via tcpdump? */
-#ifdef ENABLE_VERBOSE
-        if (options->verbose)
-            tcpdump_print(options->tcpdump, pkthdr_ptr, pktdata);
-#endif
-
         if (ctx->options->unique_ip && ctx->unique_iteration &&
                 ctx->unique_iteration > ctx->last_unique_iteration) {
             /* edit packet to ensure every pass is unique */
@@ -840,12 +840,6 @@ send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *
         if (options->flow_stats && !options->file_cache[cache_file_idx].cached)
             update_flow_stats(ctx, sp, pkthdr_ptr, pktdata, datalink);
 
-        if (ctx->first_time) {
-            /* get time and timestamp of the first packet */
-            gettimeofday(&now, NULL);
-            memcpy(&first_pkt_ts, &pkthdr_ptr->ts, sizeof(struct timeval));
-        }
-
         /*
          * this accelerator improves performance by avoiding expensive
          * time stamps during periods where we have fallen behind in our
@@ -860,74 +854,94 @@ send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *
              * time stamping is expensive, but now is the
              * time to do it.
              */
-            dbgx(4, "This packet time: " TIMEVAL_FORMAT, pkthdr_ptr->ts.tv_sec, pkthdr_ptr->ts.tv_usec);
+            dbgx(4, "This packet time: " TIMEVAL_FORMAT, pkthdr_ptr->ts.tv_sec,
+                    pkthdr_ptr->ts.tv_usec);
             skip_length = 0;
             ctx->skip_packets = 0;
 
-            /*
-             * Only sleep if we're not in top speed mode (-t)
-             * or if the current packet is not late.
-             *
-             * This also sets skip_length which will avoid timestamping for
-             * a given number of packets.
-             */
-            timersub(&pkthdr_ptr->ts, &first_pkt_ts, &pkt_ts_delta);
-            if (calc_sleep_time(ctx, &pkt_ts_delta, &ctx->stats.last_time, pktlen, sp, packetnum,
-                    &ctx->stats.end_time, &start_us, &skip_length)) {
-                now_is_now = true;
+            if (options->speed.mode == speed_multiplier) {
+                if (!timerisset(&last_pkt_ts)) {
+                    TIMEVAL_SET(&last_pkt_ts, &pkthdr_ptr->ts);
+                } else if (timercmp(&pkthdr_ptr->ts, &last_pkt_ts, >)) {
+                    struct timeval delta;
+
+                    timersub(&pkthdr_ptr->ts, &last_pkt_ts, &delta);
+                    timeradd(&stats->pkt_ts_delta, &delta, &stats->pkt_ts_delta);
+                    TIMEVAL_SET(&last_pkt_ts, &pkthdr_ptr->ts);
+                }
+
+                if (!timerisset(&stats->time_delta))
+                    TIMEVAL_SET(&stats->pkt_ts_delta, &stats->pkt_ts_delta);
+            }
+
+            if (!top_speed) {
                 gettimeofday(&now, NULL);
+                now_is_now = true;
             }
 
             /*
-             * track the time of the "last packet sent".  Again, because of OpenBSD
-             * we have to do a memcpy rather then assignment.
+             * Only if the current packet is not late.
+             *
+             * This also sets skip_length and skip_packets which will avoid
+             * timestamping for a given number of packets.
+             */
+            calc_sleep_time(ctx, &stats->pkt_ts_delta, &stats->time_delta,
+                    pktlen, sp, packetnum, &stats->end_time,
+                    TIMEVAL_TO_MICROSEC(&stats->start_time), &skip_length);
+
+            /*
+             * Track the time of the "last packet sent".
              *
              * A number of 3rd party tools generate bad timestamps which go backwards
              * in time.  Hence, don't update the "last" unless pkthdr.ts > last
              */
-            if (!top_speed && timercmp(&ctx->stats.last_time, &pkt_ts_delta, <))
-                memcpy(&ctx->stats.last_time, &pkt_ts_delta, sizeof(struct timeval));
+            if (timercmp(&stats->time_delta, &stats->pkt_ts_delta, <))
+                TIMEVAL_SET(&stats->time_delta, &stats->pkt_ts_delta);
 
             /*
              * we know how long to sleep between sends, now do it.
              */
-            if (timesisset(&ctx->nap))
-                tcpr_sleep(ctx, sp, &ctx->nap, &now, options->accurate);
+            if (!top_speed)
+                tcpr_sleep(ctx, sp, &ctx->nap, &now);
         }
 
-        dbgx(2, "Sending packet #" COUNTER_SPEC, packetnum);
+#ifdef ENABLE_VERBOSE
+        /* do we need to print the packet via tcpdump? */
+        if (options->verbose)
+            tcpdump_print(options->tcpdump, pkthdr_ptr, pktdata);
+#endif
 
+        dbgx(2, "Sending packet #" COUNTER_SPEC, packetnum);
         /* write packet out on network */
-        if (sendpacket(sp, pktdata, pktlen, pkthdr_ptr) < (int)pktlen)
+        if (sendpacket(sp, pktdata, pktlen, pkthdr_ptr) < (int)pktlen) {
             warnx("Unable to send packet: %s", sendpacket_geterr(sp));
+            break;
+        }
 
         /*
-         * mark the time when we sent the last packet
-         *
-         * we have to cast the ts, since OpenBSD sucks
-         * had to be special and use bpf_timeval.
+         * Mark the time when we sent the last packet
          */
-        memcpy(&ctx->stats.end_time, &now, sizeof(ctx->stats.end_time));
+        TIMEVAL_SET(&stats->end_time, &now);
 
-        ctx->stats.pkts_sent++;
-        ctx->stats.bytes_sent += pktlen;
+        stats->pkts_sent++;
+        stats->bytes_sent += pktlen;
 
         /* print stats during the run? */
         if (options->stats > 0) {
-            if (! timerisset(&ctx->stats.last_print)) {
-                memcpy(&ctx->stats.last_print, &now, sizeof(ctx->stats.last_print));
+            if (! timerisset(&stats->last_print)) {
+                TIMEVAL_SET(&stats->last_print, &now);
             } else {
-                timersub(&now, &ctx->stats.last_print, &print_delta);
+                timersub(&now, &stats->last_print, &print_delta);
                 if (print_delta.tv_sec >= options->stats) {
-                    memcpy(&ctx->stats.end_time, &now, sizeof(ctx->stats.end_time));
-                    packet_stats(&ctx->stats);
-                    memcpy(&ctx->stats.last_print, &now, sizeof(ctx->stats.last_print));
+                    TIMEVAL_SET(&stats->end_time, &now);
+                    packet_stats(stats);
+                    TIMEVAL_SET(&stats->last_print, &now);
                 }
             }
         }
 
 #if defined HAVE_NETMAP
-        if (sp->first_packet) {
+        if (sp->first_packet || timesisset(&ctx->nap)) {
             wake_send_queues(sp, options);
             sp->first_packet = false;
         }
@@ -941,9 +955,9 @@ send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *
         }
 
         /* stop sending based on the duration limit... */
-        if ((end_us > 0 && TIMEVAL_TO_MICROSEC(&now) > end_us) ||
+        if ((end_us > 0 && (COUNTER)TIMEVAL_TO_MICROSEC(&now) > end_us) ||
                 /* ... or stop sending based on the limit -L? */
-                (limit_send > 0 && ctx->stats.pkts_sent >= limit_send)) {
+                (limit_send > 0 && stats->pkts_sent >= limit_send)) {
             ctx->abort = true;
         }
     } /* while */
@@ -966,7 +980,7 @@ send_dual_packets(tcpreplay_t *ctx, pcap_t *pcap1, int cache_file_idx1, pcap_t *
     if (!now_is_now)
         gettimeofday(&now, NULL);
 
-    memcpy(&ctx->stats.end_time, &now, sizeof(ctx->stats.end_time));
+    TIMEVAL_SET(&stats->end_time, &now);
 
     increment_iteration(ctx);
 }
@@ -1021,7 +1035,7 @@ get_next_packet(tcpreplay_t *ctx, pcap_t *pcap, struct pcap_pkthdr *pkthdr, int
             /*
              * We should read the pcap file, and cache the results
              */
-            pktdata = (u_char *)pcap_next(pcap, pkthdr);
+            pktdata = safe_pcap_next(pcap, pkthdr);
             if (pktdata != NULL) {
                 if (*prev_packet == NULL) {
                     /*
@@ -1051,7 +1065,7 @@ get_next_packet(tcpreplay_t *ctx, pcap_t *pcap, struct pcap_pkthdr *pkthdr, int
         /*
          * Read pcap file as normal
          */
-        pktdata = (u_char *)pcap_next(pcap, pkthdr);
+        pktdata = safe_pcap_next(pcap, pkthdr);
     }
 
     /* this get's casted to a const on the way out */
@@ -1103,15 +1117,14 @@ cache_mode(tcpreplay_t *ctx, char *cachedata, COUNTER packet_num)
  * calculate the appropriate amount of time to sleep. Sleep time
  * will be in ctx->nap.
  */
-static bool calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_time_delta,
-        struct timeval *last_delta, COUNTER len,
+static void calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_ts_delta,
+        struct timeval *time_delta, COUNTER len,
         sendpacket_t *sp, COUNTER counter, timestamp_t *sent_timestamp,
-        COUNTER *start_us, COUNTER *skip_length)
+        COUNTER start_us, COUNTER *skip_length)
 {
     tcpreplay_opt_t *options = ctx->options;
     struct timeval nap_for;
     COUNTER now_us;
-    bool update_time = true;
 
     timesclear(&ctx->nap);
 
@@ -1126,36 +1139,24 @@ static bool calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_time_delta,
     /* no last packet sent, just leave */
     if (ctx->first_time) {
         ctx->first_time = false;
-        return false;
+        return;
     }
 
     switch(options->speed.mode) {
     case speed_multiplier:
         /*
          * Replay packets a factor of the time they were originally sent.
+         * Make sure the packet is not late.
          */
-        if (timerisset(last_delta)) {
-            /* make sure the packet is not late */
-            COUNTER delta_us, delta_pkt_time;
-            now_us = TIMSTAMP_TO_MICROSEC(sent_timestamp);
-            delta_pkt_time = TIMEVAL_TO_MICROSEC(pkt_time_delta);
-            delta_us = now_us - *start_us;
-            if (timercmp(pkt_time_delta, last_delta, >) && (delta_pkt_time > delta_us)) {
-                /* pkt_time_delta has increased, so handle normally */
-                timersub(pkt_time_delta, last_delta, &nap_for);
-                dbgx(3, "original packet delta pkt_time: " TIMEVAL_FORMAT, nap_for.tv_sec, nap_for.tv_usec);
-
-                TIMEVAL_TO_TIMESPEC(&nap_for, &ctx->nap);
-                dbgx(3, "original packet delta timv: " TIMESPEC_FORMAT, ctx->nap.tv_sec, ctx->nap.tv_nsec);
-                timesdiv_float(&ctx->nap, options->speed.multiplier);
-                dbgx(3, "original packet delta/div: " TIMESPEC_FORMAT, ctx->nap.tv_sec, ctx->nap.tv_nsec);
-            } else {
-                /* Don't sleep if this packet is late or in the past */
-                update_time = false;
-            }
-        } else {
-            /* Don't sleep if this is our first packet */
-            update_time = false;
+        if (timercmp(pkt_ts_delta, time_delta, >)) {
+            /* pkt_time_delta has increased, so handle normally */
+            timersub(pkt_ts_delta, time_delta, &nap_for);
+            TIMEVAL_TO_TIMESPEC(&nap_for, &ctx->nap);
+            dbgx(3, "original packet delta time: " TIMESPEC_FORMAT,
+                    ctx->nap.tv_sec, ctx->nap.tv_nsec);
+            timesdiv_float(&ctx->nap, options->speed.multiplier);
+            dbgx(3, "original packet delta/div: " TIMESPEC_FORMAT,
+                    ctx->nap.tv_sec, ctx->nap.tv_nsec);
         }
         break;
 
@@ -1169,7 +1170,8 @@ static bool calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_time_delta,
             COUNTER next_tx_us;
             COUNTER bps = options->speed.speed;
             COUNTER bits_sent = ((ctx->stats.bytes_sent + len) * 8);
-            COUNTER tx_us = now_us - *start_us;
+            COUNTER tx_us = now_us - start_us;
+
             /*
              * bits * 1000000 divided by bps = microseconds
              *
@@ -1179,13 +1181,16 @@ static bool calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_time_delta,
                 next_tx_us = (bits_sent * 1000) / bps * 1000;
             else
                 next_tx_us = (bits_sent * 1000000) / bps;
+
             if (next_tx_us > tx_us) {
                 NANOSEC_TO_TIMESPEC((next_tx_us - tx_us) * 1000, &ctx->nap);
             } else if (tx_us > next_tx_us) {
-                tx_us = now_us - *start_us;
+                tx_us = now_us - start_us;
                 *skip_length = ((tx_us - next_tx_us) * bps) / 8000000;
             }
-            update_current_timestamp_trace_entry(ctx->stats.bytes_sent + (COUNTER)len, now_us, tx_us, next_tx_us);
+
+            update_current_timestamp_trace_entry(ctx->stats.bytes_sent +
+                    (COUNTER)len, now_us, tx_us, next_tx_us);
         }
 
         dbgx(3, "packet size=" COUNTER_SPEC "\t\tnap=" TIMESPEC_FORMAT, len,
@@ -1199,24 +1204,27 @@ static bool calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_time_delta,
           */
          now_us = TIMSTAMP_TO_MICROSEC(sent_timestamp);
          if (now_us) {
-             COUNTER pph = ctx->options->speed.speed * (ctx->options->speed.pps_multi > 0 ? ctx->options->speed.pps_multi : (60 * 60));
+             COUNTER pph = ctx->options->speed.speed *
+                     (ctx->options->speed.pps_multi > 0 ? ctx->options->speed.pps_multi : (60 * 60));
              COUNTER pkts_sent = ctx->stats.pkts_sent;
              /*
               * packets * 1000000 divided by pps = microseconds
               * packets per sec (pps) = packets per hour / (60 * 60)
               */
              COUNTER next_tx_us = (pkts_sent * 1000000) * (60 * 60) / pph;
-             COUNTER tx_us = now_us - *start_us;
+             COUNTER tx_us = now_us - start_us;
+
              if (next_tx_us > tx_us)
                  NANOSEC_TO_TIMESPEC((next_tx_us - tx_us) * 1000, &ctx->nap);
              else
                  ctx->skip_packets = options->speed.pps_multi;
 
-             update_current_timestamp_trace_entry(ctx->stats.bytes_sent + (COUNTER)len, now_us, tx_us, next_tx_us);
+             update_current_timestamp_trace_entry(ctx->stats.bytes_sent +
+                     (COUNTER)len, now_us, tx_us, next_tx_us);
          }
 
-         dbgx(3, "packet count=" COUNTER_SPEC "\t\tnap=" TIMESPEC_FORMAT, ctx->stats.pkts_sent,
-                 ctx->nap.tv_sec, ctx->nap.tv_nsec);
+         dbgx(3, "packet count=" COUNTER_SPEC "\t\tnap=" TIMESPEC_FORMAT,
+                 ctx->stats.pkts_sent, ctx->nap.tv_sec, ctx->nap.tv_nsec);
         break;
 
     case speed_oneatatime:
@@ -1240,13 +1248,10 @@ static bool calc_sleep_time(tcpreplay_t *ctx, struct timeval *pkt_time_delta,
         errx(-1, "Unknown/supported speed mode: %d", options->speed.mode);
         break;
     }
-
-    return update_time;
 }
 
 static void tcpr_sleep(tcpreplay_t *ctx, sendpacket_t *sp,
-        struct timespec *nap_this_time, struct timeval *now,
-        tcpreplay_accurate accurate)
+        struct timespec *nap_this_time, struct timeval *now)
 {
     tcpreplay_opt_t *options = ctx->options;
     bool flush =
@@ -1266,9 +1271,12 @@ static void tcpr_sleep(tcpreplay_t *ctx, sendpacket_t *sp,
         dbgx(2, "Was going to sleep for " TIMESPEC_FORMAT " but maxsleeping for " TIMESPEC_FORMAT,
             nap_this_time->tv_sec, nap_this_time->tv_nsec, options->maxsleep.tv_sec,
             options->maxsleep.tv_nsec);
-        memcpy(nap_this_time, &(options->maxsleep), sizeof(*nap_this_time));
+        TIMESPEC_SET(nap_this_time, &options->maxsleep);
     }
 
+    if (flush)
+        wake_send_queues(sp, options);
+
     dbgx(2, "Sleeping:                   " TIMESPEC_FORMAT,
             nap_this_time->tv_sec, nap_this_time->tv_nsec);
 
@@ -1276,11 +1284,16 @@ static void tcpr_sleep(tcpreplay_t *ctx, sendpacket_t *sp,
      * Depending on the accurate method & packet rate computation method
      * We have multiple methods of sleeping, pick the right one...
      */
-    switch (accurate) {
+    switch (options->accurate) {
 #ifdef HAVE_SELECT
     case accurate_select:
-        select_sleep(nap_this_time);
-        gettimeofday(now, NULL);
+        select_sleep(sp, nap_this_time, now, flush);
+        break;
+#endif
+
+#if defined HAVE_IOPORT_SLEEP
+    case accurate_ioport:
+        ioport_sleep(sp, nap_this_time, now, flush);
         break;
 #endif
 
@@ -1289,12 +1302,11 @@ static void tcpr_sleep(tcpreplay_t *ctx, sendpacket_t *sp,
         break;
 
     case accurate_nanosleep:
-        nanosleep_sleep(nap_this_time);
-        gettimeofday(now, NULL);
+        nanosleep_sleep(sp, nap_this_time, now, flush);
         break;
 
     default:
-        errx(-1, "Unknown timer mode %d", accurate);
+        errx(-1, "Unknown timer mode %d", options->accurate);
     }
 }
 
