@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2016 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2018 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -38,18 +38,18 @@
 int 
 tcpedit_post_args(tcpedit_t *tcpedit) {
     int rcode = 0;
-    long ttl;
+    int i;
+    uint32_t seed = 1, rand_num;
 
     assert(tcpedit);
 
-
     /* --pnat */
     if (HAVE_OPT(PNAT)) {
         int ct = STACKCT_OPT(PNAT);
         char **list = (char**)STACKLST_OPT(PNAT);
         int first = 1;
 
-        tcpedit->rewrite_ip ++;
+        tcpedit->rewrite_ip = true;
 
         do {
             char *p = *list++;
@@ -73,7 +73,7 @@ tcpedit_post_args(tcpedit_t *tcpedit) {
     
     /* --srcipmap */
     if (HAVE_OPT(SRCIPMAP)) {
-        tcpedit->rewrite_ip ++;
+        tcpedit->rewrite_ip = true;
         if (! parse_cidr_map(&tcpedit->srcipmap, OPT_ARG(SRCIPMAP))) {
             tcpedit_seterr(tcpedit, 
                 "Unable to parse --srcipmap=%s", OPT_ARG(SRCIPMAP));
@@ -83,7 +83,7 @@ tcpedit_post_args(tcpedit_t *tcpedit) {
 
     /* --dstipmap */
     if (HAVE_OPT(DSTIPMAP)) {
-        tcpedit->rewrite_ip ++;
+        tcpedit->rewrite_ip = true;
         if (! parse_cidr_map(&tcpedit->dstipmap, OPT_ARG(DSTIPMAP))) {
             tcpedit_seterr(tcpedit, 
                 "Unable to parse --dstipmap=%s", OPT_ARG(DSTIPMAP));
@@ -108,6 +108,8 @@ tcpedit_post_args(tcpedit_t *tcpedit) {
 
     /* --ttl */
     if (HAVE_OPT(TTL)) {
+        long ttl;
+
         if (strchr(OPT_ARG(TTL), '+')) {
             tcpedit->ttl_mode = TCPEDIT_TTL_MODE_ADD;            
         } else if (strchr(OPT_ARG(TTL), '-')) {
@@ -166,6 +168,16 @@ tcpedit_post_args(tcpedit_t *tcpedit) {
         }
     }
 
+    /* --rewrite-sequence */
+    if (HAVE_OPT(TCP_SEQUENCE)) {
+        tcpedit->tcp_sequence_enable = 1;
+        seed = OPT_VALUE_TCP_SEQUENCE;
+        for (i = 0; i < 5; ++i)
+            rand_num = tcpr_random(&seed);
+
+        tcpedit->tcp_sequence_adjust = rand_num;
+    }
+
     /* TCP/UDP port rewriting */
     if (HAVE_OPT(PORTMAP)) {
         int ct = STACKCT_OPT(PORTMAP);
@@ -198,15 +210,32 @@ tcpedit_post_args(tcpedit_t *tcpedit) {
     }
 
     /*
-     * IP address rewriting processing.  Call srandom() then add up
-     * 5 calls to random() as our mixer for randomizing.  This should
+     * IP address rewriting processing. Update seed by making
+     * 5 calls to tcpr_random() as our mixer for randomizing.  This should
      * work better since most people aren't going to write out values
      * close to 32bit integers.
      */
     if (HAVE_OPT(SEED)) {
         tcpedit->rewrite_ip = true;
-        srandom(OPT_VALUE_SEED);
-        tcpedit->seed = random() + random() + random() + random() + random();
+        seed = OPT_VALUE_SEED;
+    } else if (HAVE_OPT(FUZZ_SEED)) {
+        /* --fuzz-seed */
+        seed = OPT_VALUE_FUZZ_SEED;
+        tcpedit->fuzz_factor = OPT_VALUE_FUZZ_FACTOR;
+    }
+
+    for (i = 0; i < 5; ++i)
+        rand_num = tcpr_random(&seed);
+
+    srandom(rand_num);
+    if (HAVE_OPT(SEED)) {
+        tcpedit->rewrite_ip = true;
+        tcpedit->seed = seed;
+    }
+
+    if (HAVE_OPT(FUZZ_SEED)) {
+        /* --fuzz-seed */
+        tcpedit->fuzz_seed = seed;
     }
 
     if (HAVE_OPT(ENDPOINTS)) {
@@ -219,28 +248,6 @@ tcpedit_post_args(tcpedit_t *tcpedit) {
         }
     }
 
-    /* 
-     * figure out the max packet len
-    if (tcpedit->l2.enabled) {
-        // custom l2 header
-        dbg(1, "Using custom L2 header to calculate max frame size\n");
-        tcpedit->maxpacket = tcpedit->mtu + tcpedit->l2.len;
-    }
-    else if (tcpedit->l2.dlt == DLT_EN10MB || tcpedit->l2.dlt == DLT_VLAN) {
-        // ethernet 
-        dbg(1, "Using Ethernet to calculate max frame size\n");
-        tcpedit->maxpacket = tcpedit->mtu + TCPR_ETH_H;
-    } else {
-         // uh, wtf is this now?  we'll just assume ethernet and hope things work
-        tcpedit->maxpacket = tcpedit->mtu + TCPR_ETH_H;
-        tcpedit_seterr(tcpedit, 
-            "Unsupported DLT type: %s.  We'll just treat it as ethernet.\n"
-            "You may need to increase the MTU (-t <size>) if you get errors\n",
-            pcap_datalink_val_to_name(tcpedit->l2.dlt));
-        rcode = 1;
-    }
-     */
-     
     /* parse the tcpedit dlt args */
     rcode = tcpedit_dlt_post_args(tcpedit);
     if (rcode < 0) {
