@@ -2,7 +2,7 @@
 
 /*
  * Copyright (c) 2006-2007 Aaron Turner.
- * Copyright (c) 2013-2016 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ * Copyright (c) 2013-2018 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -116,13 +116,9 @@ dlt_jnpr_ether_init(tcpeditdlt_t *ctx)
         return TCPEDIT_ERROR;
     }
     
-    /* allocate memory for our deocde extra data */
-    if (sizeof(jnpr_ether_extra_t) > 0)
-        ctx->decoded_extra = safe_malloc(sizeof(jnpr_ether_extra_t));
-
     /* allocate memory for our config data */
-    if (sizeof(jnpr_ether_config_t) > 0)
-        plugin->config = safe_malloc(sizeof(jnpr_ether_config_t));
+    plugin->config_size = sizeof(jnpr_ether_config_t);
+    plugin->config = safe_malloc(plugin->config_size);
 
     return TCPEDIT_OK; /* success */
 }
@@ -156,7 +152,6 @@ int
 dlt_jnpr_ether_cleanup(tcpeditdlt_t *ctx)
 {
     tcpeditdlt_plugin_t *plugin;
-    jnpr_ether_config_t *config;
     
     assert(ctx);
 
@@ -168,13 +163,17 @@ dlt_jnpr_ether_cleanup(tcpeditdlt_t *ctx)
     if (ctx->decoded_extra != NULL) {
         safe_free(ctx->decoded_extra);
         ctx->decoded_extra = NULL;
+        ctx->decoded_extra_size = 0;
     }
     
     if (plugin->config != NULL) {
         /* clean up the en10mb plugin */
+        jnpr_ether_config_t * config;
+
         config = (jnpr_ether_config_t *)ctx->encoder->config;
         tcpedit_dlt_cleanup(config->subctx);
         plugin->config = NULL;
+        plugin->config_size = 0;
     }
 
     return TCPEDIT_OK; /* success */
@@ -214,7 +213,10 @@ dlt_jnpr_ether_decode(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
     
     assert(ctx);
     assert(packet);
-    assert(pktlen > JUNIPER_ETHER_HEADER_LEN); /* MAGIC + Static fields + Extension Length */
+
+    /* MAGIC + Static fields + Extension Length */
+    if (pktlen < JUNIPER_ETHER_HEADER_LEN)
+        return TCPEDIT_ERROR;
 
     config = (jnpr_ether_config_t *)ctx->encoder->config;
 
@@ -272,8 +274,11 @@ int
 dlt_jnpr_ether_encode(tcpeditdlt_t *ctx, u_char *packet, int pktlen, _U_ tcpr_dir_t dir)
 {
     assert(ctx);
-    assert(pktlen > JUNIPER_ETHER_HEADER_LEN); /* MAGIC + Static fields + Extension Length */
     assert(packet);
+
+    /* MAGIC + Static fields + Extension Length */
+    if (pktlen < JUNIPER_ETHER_HEADER_LEN)
+        return TCPEDIT_ERROR;
     
     tcpedit_seterr(ctx->tcpedit, "%s", "DLT_JUNIPER_ETHER plugin does not support packet encoding");
     return TCPEDIT_ERROR;
@@ -292,7 +297,10 @@ dlt_jnpr_ether_proto(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
 
     assert(ctx);
     assert(packet);
-    assert(pktlen > JUNIPER_ETHER_HEADER_LEN); /* MAGIC + Static fields + Extension Length */
+
+    /* MAGIC + Static fields + Extension Length */
+    if (pktlen < JUNIPER_ETHER_HEADER_LEN)
+        return TCPEDIT_ERROR;
 
     config = (jnpr_ether_config_t *)ctx->encoder->config;
 
@@ -334,8 +342,8 @@ dlt_jnpr_ether_get_layer3(tcpeditdlt_t *ctx, u_char *packet, const int pktlen)
     }
 
     l2len = dlt_jnpr_ether_l2len(ctx, packet, pktlen);
-
-    assert(pktlen >= l2len);
+    if (l2len == -1 || pktlen < l2len)
+        return NULL;
 
     return tcpedit_dlt_l3data_copy(ctx, packet, pktlen, l2len);
 }
@@ -355,8 +363,8 @@ dlt_jnpr_ether_merge_layer3(tcpeditdlt_t *ctx, u_char *packet, const int pktlen,
     assert(l3data);
     
     l2len = dlt_jnpr_ether_l2len(ctx, packet, pktlen);
-    
-    assert(pktlen >= l2len);
+    if (l2len == -1 || pktlen < l2len)
+        return NULL;
     
     return tcpedit_dlt_l3data_merge(ctx, packet, pktlen, l3data, l2len);
 }
@@ -374,7 +382,9 @@ dlt_jnpr_ether_get_mac(tcpeditdlt_t *ctx, tcpeditdlt_mac_type_t mac, const u_cha
 
     assert(ctx);
     assert(packet);
-    assert(pktlen);
+
+    if (pktlen < JUNIPER_ETHER_EXTLEN_OFFSET + 2)
+        return NULL;
 
     config = (jnpr_ether_config_t *)ctx->encoder->config;
 
@@ -394,12 +404,14 @@ dlt_jnpr_ether_get_mac(tcpeditdlt_t *ctx, tcpeditdlt_mac_type_t mac, const u_cha
 int
 dlt_jnpr_ether_l2len(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
 {
-    uint16_t len;
+    int len, res;
     jnpr_ether_config_t *config;
 
     assert(ctx);
     assert(packet);
-    assert(pktlen);
+
+    if (pktlen < JUNIPER_ETHER_EXTLEN_OFFSET + 2)
+        return -1;
     
     config = (jnpr_ether_config_t *)ctx->encoder->config;
 
@@ -410,7 +422,11 @@ dlt_jnpr_ether_l2len(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
     dbgx(3, "juniper header len: %u", len);
     
     /* add the 802.3 length */
-    len += tcpedit_dlt_l2len(config->subctx, DLT_EN10MB, (packet + len), (pktlen - len));
+    res = tcpedit_dlt_l2len(config->subctx, DLT_EN10MB, (packet + len), (pktlen - len));
+    if (res == -1)
+        return TCPEDIT_ERROR;
+
+    len += res;
     dbgx(3, "total l2len: %u", len);
     
     /* and return that */
