@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2017 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2018 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -37,7 +37,7 @@
 #include "edit_packet.h"
 #include "parse_args.h"
 #include "fuzzing.h"
-
+#include "rewrite_sequence.h"
 
 #include "lib/sll.h"
 #include "dlt.h"
@@ -87,31 +87,37 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
     ipv4_hdr_t *ip_hdr = NULL;
     ipv6_hdr_t *ip6_hdr = NULL;
     arp_hdr_t *arp_hdr = NULL;
-    int l2len = 0, l2proto, retval = 0, dst_dlt, src_dlt, pktlen, lendiff;
+    int l2len, l2proto, retval = 0;
+    int dst_dlt, src_dlt, pktlen, lendiff;
     int ipflags = 0, tclass = 0;
     int needtorecalc = 0;           /* did the packet change? if so, checksum */
-    u_char *packet = *pktdata;
+    u_char *packet;
+
     assert(tcpedit);
     assert(pkthdr);
     assert(*pkthdr);
     assert(pktdata);
-    assert(packet);
+    assert(*pktdata);
     assert(tcpedit->validated);
 
+    packet = *pktdata;
 
     tcpedit->runtime.packetnum++;
+
     dbgx(3, "packet " COUNTER_SPEC " caplen %d", 
             tcpedit->runtime.packetnum, (*pkthdr)->caplen);
 
-
     /*
      * remove the Ethernet FCS (checksum)?
      * note that this feature requires the end user to be smart and
      * only set this flag IFF the pcap has the FCS.  If not, then they
      * just removed 2 bytes of ACTUAL PACKET DATA.  Sucks to be them.
      */
-    if (tcpedit->efcs > 0) {
-        (*pkthdr)->caplen -= 4;
+    if (tcpedit->efcs > 0 &&(*pkthdr)->len > 4) {
+        if ((*pkthdr)->caplen == (*pkthdr)->len) {
+            (*pkthdr)->caplen -= 4;
+        }
+
         (*pkthdr)->len -= 4;
     }
 
@@ -141,48 +147,87 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
     
     dst_dlt = tcpedit_dlt_dst(tcpedit->dlt_ctx);
     l2len = tcpedit_dlt_l2len(tcpedit->dlt_ctx, dst_dlt, packet, (*pkthdr)->caplen);
+    if (l2len == -1)
+        return TCPEDIT_ERROR;
 
     dbgx(2, "dst_dlt = %04x\tsrc_dlt = %04x\tproto = %04x\tl2len = %d", dst_dlt, src_dlt, ntohs(l2proto), l2len);
 
     /* does packet have an IP header?  if so set our pointer to it */
     if (l2proto == htons(ETHERTYPE_IP)) {
-        ip_hdr = (ipv4_hdr_t *)tcpedit_dlt_l3data(tcpedit->dlt_ctx, dst_dlt, packet, (*pkthdr)->caplen);
-        if (ip_hdr == NULL) {
-            return TCPEDIT_ERROR;
-        }        
-        dbgx(3, "Packet has an IPv4 header: %p...", ip_hdr);
-    } else if (l2proto == htons(ETHERTYPE_IP6)) {
-        ip6_hdr = (ipv6_hdr_t *)tcpedit_dlt_l3data(tcpedit->dlt_ctx, dst_dlt, packet, (*pkthdr)->caplen);
-        if (ip6_hdr == NULL) {
+        u_char *p;
+
+        if ((*pkthdr)->caplen < l2len + sizeof(*ip_hdr)) {
+            tcpedit_seterr(tcpedit, "Packet length %d is to short to contain a layer IP header for DLT 0x%04x",
+                    pktlen, dst_dlt);
             return TCPEDIT_ERROR;
         }
-        dbgx(3, "Packet has an IPv6 header: %p...", ip6_hdr);
+
+        ip_hdr = (ipv4_hdr_t *)tcpedit_dlt_l3data(tcpedit->dlt_ctx, dst_dlt, packet, (*pkthdr)->caplen);
+        if (ip_hdr == NULL)
+            return TCPEDIT_ERROR;
+
+        p = get_layer4_v4(ip_hdr, (*pkthdr)->caplen - l2len);
+        if (!p) {
+            tcpedit_seterr(tcpedit, "Packet length %d is to short to contain a layer %d byte IP header for DLT 0x%04x",
+                    pktlen, ip_hdr->ip_hl << 2,  dst_dlt);
+            return TCPEDIT_ERROR;
+        }
+
+        dbgx(3, "Packet has an IPv4 header: 0x%p...", ip_hdr);
+    } else if (l2proto == htons(ETHERTYPE_IP6)) {
+        u_char *p;
+
+        if ((*pkthdr)->caplen < l2len + sizeof(*ip6_hdr)) {
+            tcpedit_seterr(tcpedit, "Packet length %d is to short to contain a layer IPv6 header for DLT 0x%04x",
+                    pktlen, dst_dlt);
+            return TCPEDIT_ERROR;
+        }
+
+        ip6_hdr = (ipv6_hdr_t *)tcpedit_dlt_l3data(tcpedit->dlt_ctx, dst_dlt, packet, (*pkthdr)->caplen);
+        if (ip6_hdr == NULL)
+            return TCPEDIT_ERROR;
+
+        p = get_layer4_v6(ip6_hdr, (*pkthdr)->caplen - l2len);
+        if (!p) {
+            tcpedit_seterr(tcpedit, "Packet length %d is to short to contain an IPv6 header for DLT 0x%04x",
+                    pktlen, dst_dlt);
+            return TCPEDIT_ERROR;
+        }
+
+        dbgx(3, "Packet has an IPv6 header: 0x%p...", ip6_hdr);
     } else {
         dbgx(3, "Packet isn't IPv4 or IPv6: 0x%04x", l2proto);
         /* non-IP packets have a NULL ip_hdr struct */
         ip_hdr = NULL;
+        ip6_hdr = NULL;
     }
 
     /* The following edits only apply for IPv4 */
     if (ip_hdr != NULL) {
-        
+
         /* set TOS ? */
         if (tcpedit->tos > -1) {
-            uint16_t newval, oldval = *((uint16_t*)ip_hdr);
+            volatile uint16_t oldval = *((uint16_t*)ip_hdr);
+            volatile uint16_t newval;
+
             ip_hdr->ip_tos = tcpedit->tos;
             newval = *((uint16_t*)ip_hdr);
             csum_replace2(&ip_hdr->ip_sum, oldval, newval);
         }
-            
+
         /* rewrite the TTL */
         rewrite_ipv4_ttl(tcpedit, ip_hdr);
 
         /* rewrite TCP/UDP ports */
         if (tcpedit->portmap != NULL) {
-            if ((retval = rewrite_ipv4_ports(tcpedit, &ip_hdr)) < 0)
+            if ((retval = rewrite_ipv4_ports(tcpedit, &ip_hdr, (*pkthdr)->caplen)) < 0)
                 return TCPEDIT_ERROR;
         }
+
+        if (tcpedit->tcp_sequence_enable)
+            rewrite_ipv4_sequence(tcpedit, &ip_hdr);
     }
+
     /* IPv6 edits */
     else if (ip6_hdr != NULL) {
         /* rewrite the hop limit */
@@ -216,9 +261,12 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
 
         /* rewrite TCP/UDP ports */
         if (tcpedit->portmap != NULL) {
-            if ((retval = rewrite_ipv6_ports(tcpedit, &ip6_hdr)) < 0)
+            if ((retval = rewrite_ipv6_ports(tcpedit, &ip6_hdr, (*pkthdr)->caplen)) < 0)
                 return TCPEDIT_ERROR;
         }
+
+        if (tcpedit->tcp_sequence_enable)
+            rewrite_ipv6_sequence(tcpedit, &ip6_hdr);
     }
 
     if (tcpedit->fuzz_seed != 0) {
@@ -240,10 +288,12 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
     if (tcpedit->rewrite_ip) {
         /* IP packets */
         if (ip_hdr != NULL) {
-            if ((retval = rewrite_ipv4l3(tcpedit, ip_hdr, direction)) < 0)
+            if ((retval = rewrite_ipv4l3(tcpedit, ip_hdr, direction,
+                    (*pkthdr)->caplen - l2len)) < 0)
                 return TCPEDIT_ERROR;
         } else if (ip6_hdr != NULL) {
-            if ((retval = rewrite_ipv6l3(tcpedit, ip6_hdr, direction)) < 0)
+            if ((retval = rewrite_ipv6l3(tcpedit, ip6_hdr, direction,
+                    (*pkthdr)->caplen - l2len)) < 0)
                 return TCPEDIT_ERROR;
         }
 
@@ -265,12 +315,12 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
         /* IPv4 Packets */
         if (ip_hdr != NULL) {
             if ((retval = randomize_ipv4(tcpedit, *pkthdr, packet, 
-                    ip_hdr)) < 0)
+                    ip_hdr, (*pkthdr)->caplen - l2len)) < 0)
                 return TCPEDIT_ERROR;
 
         } else if (ip6_hdr != NULL) {
             if ((retval = randomize_ipv6(tcpedit, *pkthdr, packet,
-                    ip6_hdr)) < 0)
+                    ip6_hdr, (*pkthdr)->caplen - l2len)) < 0)
                 return TCPEDIT_ERROR;
 
         /* ARP packets */
@@ -287,6 +337,16 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
         }
     }
 
+    /*
+     * fix IP packet lengths in case they are corrupted by TCP segmentation
+     * offload
+     */
+    if (ip_hdr != NULL) {
+        fix_ipv4_length(*pkthdr, ip_hdr);
+    } else if (ip6_hdr != NULL) {
+        fix_ipv6_length(*pkthdr, ip6_hdr);
+    }
+
     /* do we need to fix checksums? -- must always do this last! */
     if ((tcpedit->fixcsum || needtorecalc)) {
         if (ip_hdr != NULL) {
@@ -531,21 +591,6 @@ tcpedit_l3data(tcpedit_t *tcpedit, tcpedit_coder code, u_char *packet, const int
     return result;
 }
 
-/**
- * return the length of the layer 2 header.  Returns TCPEDIT_ERROR on error
- */
-int 
-tcpedit_l2len(tcpedit_t *tcpedit, tcpedit_coder code, u_char *packet, const int pktlen)
-{
-    int result = 0;
-    if (code == BEFORE_PROCESS) {
-        result = tcpedit_dlt_l2len(tcpedit->dlt_ctx, tcpedit->dlt_ctx->decoder->dlt, packet, pktlen);
-    } else {
-        result = tcpedit_dlt_l2len(tcpedit->dlt_ctx, tcpedit->dlt_ctx->encoder->dlt, packet, pktlen);
-    }
-    return result;
-}
-
 /**
  * Returns the layer 3 type, often encoded as the layer2.proto field
  */
@@ -560,24 +605,3 @@ tcpedit_l3proto(tcpedit_t *tcpedit, tcpedit_coder code, const u_char *packet, co
     }
     return ntohs(result);
 }
-
-/*
-u_char *
-tcpedit_srcmac(tcpedit_t *tcpedit, tcpedit_coder code, u_char *packet, const int pktlen)
-{
-   
-}
-
-u_char *
-tcpedit_dstmac(tcpedit_t *tcpedit, tcpedit_coder code, u_char *packet, const int pktlen)
-{
-    
-}
-
-int 
-tcpedit_maclen(tcpedit_t *tcpedit, tcpedit_coder code)
-{
-    
-}
-
-*/
