@@ -158,6 +158,7 @@ main(int argc, char **argv)
     pcap_t *local_handle;
     char errbuf[PCAP_ERRBUF_SIZE];
     char ebuf[SENDPACKET_ERRBUF_SIZE];
+    int i;
 
     optionProcess(&tcpliveplayOptions, argc, argv); /*Process AutoOpts for manpage options*/
 
@@ -208,7 +209,7 @@ main(int argc, char **argv)
     /* Rewrites the given "*.pcap" file with all the new parameters and returns the number of packets */
     /* that need to be replayed */
     num_packets = rewrite(&new_remoteip, &new_remotemac, &myip, &mymac, argv[2], new_src_port);
-    if (num_packets < 0)
+    if (num_packets < 2)
         errx(-1, "Unable to rewrite PCAP file %s\n",argv[2]);
 
     /* create schedule & set it up */
@@ -219,8 +220,13 @@ main(int argc, char **argv)
     pkts_scheduled = setup_sched(sched);    /* Returns number of packets in schedule*/
 
     /* Set up the schedule struct to be relative numbers rather than absolute*/
+    for (i = 0; i < num_packets; i++) {
+        sched[i].exp_rseq = 0;
+        sched[i].exp_rack = 0;
+    }
+
     relative_sched(sched, sched[1].exp_rseq, num_packets);
-    printf("Packets Scheduled %d\n", pkts_scheduled);
+    printf("Packets Scheduled %u\n", pkts_scheduled);
 
     /* Open socket for savedfile traffic to be sent*/
     local_handle = pcap_open_offline("newfile.pcap", errbuf);   /*call pcap library function*/
@@ -233,7 +239,7 @@ main(int argc, char **argv)
     /* Open socket for live traffic to be listed to*/
     live_handle = set_live_filter(iface, &myip, new_src_port);  /* returns a pcap_t that filters out traffic other than TCP*/
     if (live_handle == NULL) {
-        fprintf(stderr,"Error occured while listing on traffic: %s\n", errbuf);
+        fprintf(stderr,"Error occurred while listing on traffic: %s\n", errbuf);
         free(sched);
         return(2);
     }
@@ -253,7 +259,7 @@ main(int argc, char **argv)
     /* Start replay by sending the first packet, the SYN, from the schedule */
     else if(sched[0].local){  /* Send first packet*/
         sendpacket(sp, sched[sched_index].packet_ptr, sched[sched_index].pkthdr.len, &sched[sched_index].pkthdr);
-        printf("Sending Local Packet...............	[%d]\n",sched_index+1);
+        printf("Sending Local Packet...............	[%u]\n",sched_index+1);
         sched_index++; /* Proceed in the schedule */
     }
 
@@ -290,8 +296,8 @@ main(int argc, char **argv)
         } */
         /* Do the following if we receive a packet that ACKs for the same ACKing of next packet */
         else if((tcphdr_rprev->th_seq==htonl(sched[sched_index].exp_rseq)) && (tcphdr_rprev->th_ack==htonl(sched[sched_index].exp_rack)) && (size_payload_prev>0)){
-            printf("Received Remote Packet...............	[%d]\n",sched_index+1);
-            printf("Skipping Packet......................	[%d] to Packet [%d]\n",sched_index+1, sched_index+2);
+            printf("Received Remote Packet...............	[%u]\n",sched_index+1);
+            printf("Skipping Packet......................	[%u] to Packet [%u]\n",sched_index+1, sched_index+2);
             printf("Next Remote Packet Expectation met.\nProceeding in replay...\n");
             sched_index++;   
         } 
@@ -299,7 +305,7 @@ main(int argc, char **argv)
         else if(different_payload){
             printf("\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
             printf("+ WARNING: Remote host is not meeting packet size expectations.               +\n"); 
-            printf("+ for packet %-d. Application layer data differs from capture being replayed.  +\n", diff_payload_index+1);
+            printf("+ for packet %-u. Application layer data differs from capture being replayed.  +\n", diff_payload_index+1);
             printf("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n\n"); 
             printf("Requesting retransmission.\n Proceeding...\n");
             different_payload = false;  
@@ -309,7 +315,7 @@ main(int argc, char **argv)
         if(sched[sched_index].local) {
             /*Reset alarm timeout*/
             alarm (ALARM_TIMEOUT);
-            printf("Sending Local Packet...............	[%d]\n",sched_index+1); 
+            printf("Sending Local Packet...............	[%u]\n", sched_index + 1);
           
             /* edit each packet tcphdr before sending based on the schedule*/  
             if(sched_index>0){ 
@@ -320,7 +326,7 @@ main(int argc, char **argv)
             /* If 3 attempts of resending was made, then error out to the user */
             if(sched[sched_index].sent_counter==3){
                 printf("\n++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
-                printf("+ ERROR: Re-sent packet [%-d] 3 times, but remote host is not  +\n", sched_index+1); 
+                printf("+ ERROR: Re-sent packet [%-u] 3 times, but remote host is not  +\n", sched_index+1);
                 printf("+ responding as expected. 3 resend attempts are a maximum.     +\n");
                 printf("+ Closing replay...                                            +\n");
                 printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n\n");
@@ -358,25 +364,26 @@ main(int argc, char **argv)
     /* User Debug Result Printouts*/
     if(sched_index==pkts_scheduled){
         printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n"); 
-        printf("~ CONGRATS!!! You have successfully Replayed your pcap file '%s'  \n", argv[2]);
+        printf("~ CONGRATS!!! You have successfully Replayed your pcap file '%s'\n", argv[2]);
         printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n");
     }
     else {
         printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n"); 
-        printf("~ Unfortunately an error has occurred  halting the replay of  \n");
-        printf("~ the pcap file '%s'. Please see error above for details...   \n", argv[2]);
+        printf("~ Unfortunately an error has occurred  halting the replay of\n");
+        printf("~ the pcap file '%s'. Please see error above for details...\n", argv[2]);
         printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n");
     }
 
     printf("----------------TCP Live Play Summary----------------\n");
-    printf("- Packets Scheduled to be Sent & Received:          %-d   \n", pkts_scheduled);
-    printf("- Actual Packets Sent & Received:                   %-d   \n", sched_index);
-    printf("- Total Local Packet Re-Transmissions due to packet       \n");
-    printf("- loss and/or differing payload size than expected: %-d   \n", retransmissions);
-    printf("- Thank you for Playing, Play again!                      \n");
+    printf("- Packets Scheduled to be Sent & Received:          %-u\n", pkts_scheduled);
+    printf("- Actual Packets Sent & Received:                   %-u\n", sched_index);
+    printf("- Total Local Packet Re-Transmissions due to packet\n");
+    printf("- loss and/or differing payload size than expected: %-u\n", retransmissions);
+    printf("- Thank you for Playing, Play again!\n");
     printf("----------------------------------------------------------\n\n");
 
     free(sched);
+    restore_stdin();
     return 0;
 } 
 /*end of main() function*/
@@ -463,14 +470,15 @@ setup_sched(struct tcp_sched* sched){
 
     input_addr sip, dip;  /* Source & Destination IP */ 
     input_addr local_ip, remote_ip;    /* ip address of client and server*/
-    /*temporary packet buffers*/
-    struct pcap_pkthdr header;  // The header that pcap gives us
-    const u_char *packet;       // The actual packet
     pcap_t *local_handle;
+    const u_char *packet;       /* The actual packet */
+    unsigned int flags=0;
+    struct pcap_pkthdr header;  // The header that pcap gives us
     unsigned int pkt_counter=0;
-    ether_hdr *etherhdr = NULL;
-    tcp_hdr *tcphdr = NULL;
-    ipv4_hdr *iphdr = NULL;
+    bool remote = false;    /* flags to test if data is from 'client'=local or 'server'=remote */
+    bool local = false;
+    unsigned int i = 0;
+
     local_ip.byte1=0; 
     local_ip.byte2=0; 
     local_ip.byte3=0; 
@@ -481,13 +489,7 @@ setup_sched(struct tcp_sched* sched){
     remote_ip.byte3=0; 
     remote_ip.byte4=0; 
 
-    unsigned int size_ip, i = 0; 
-    unsigned int size_tcp; 
-    unsigned int size_payload; 
     char errbuf[PCAP_ERRBUF_SIZE];
-    unsigned int flags=0; 
-    bool remote = false;    /* flags to test if data is from 'cleint'=local or 'server'=remote */
-    bool local = false; 
 
 
     local_handle = pcap_open_offline("newfile.pcap", errbuf);   /*call pcap library function*/
@@ -498,12 +500,18 @@ setup_sched(struct tcp_sched* sched){
     }
 
     /*Before sending any packet, setup the schedule with the proper parameters*/
-    while((packet = pcap_next(local_handle,&header))) {
+    while((packet = safe_pcap_next(local_handle, &header))) {
+        /*temporary packet buffers*/
+        ether_hdr *etherhdr;
+        tcp_hdr *tcphdr;
+        ipv4_hdr *iphdr;
+        unsigned int size_ip;
+        unsigned int size_tcp;
+        unsigned int size_payload;
+
         pkt_counter++; /*increment number of packets seen*/
 
         memcpy(&sched[i].pkthdr, &header, sizeof(struct pcap_pkthdr));
-        //sched[i].len = header.len; 
-        //sched[i].caplen = header.caplen; 
         sched[i].packet_ptr = safe_malloc(sched[i].pkthdr.len);
         memcpy(sched[i].packet_ptr, packet, sched[i].pkthdr.len);
 
@@ -604,7 +612,6 @@ setup_sched(struct tcp_sched* sched){
             //printf("REMOTE --------%d\n",i+1);
         }
 
-
         i++; /* increment schedule index */
 
     } /*end internal loop for reading packets (all in one file)*/
@@ -705,21 +712,17 @@ set_offline_filter(char* file)
  * This function is called every time we receive a remote packet
  */
 void 
-got_packet(u_char *args, const struct pcap_pkthdr *header, const u_char *packet){
-
-    ether_hdr *etherhdr = NULL; 
-    tcp_hdr *tcphdr = NULL;
-    ipv4_hdr *iphdr = NULL;
+got_packet(_U_ u_char *args, _U_ const struct pcap_pkthdr *header,
+        const u_char *packet)
+{
+    ether_hdr *etherhdr;
+    tcp_hdr *tcphdr;
+    ipv4_hdr *iphdr;
 
     unsigned int size_ip, size_tcp, size_payload;
     unsigned int flags = 0;
 
-    /* This is to get rid of the warning */
-    args = NULL;
-    if(args == NULL) header = NULL;
-    if(header == NULL) args = NULL; 
-
-    /* Extract and examine recieved packet headers */
+    /* Extract and examine received packet headers */
     etherhdr = (ether_hdr*)(packet);
     iphdr = (ipv4_hdr *)(packet + SIZE_ETHERNET);
     size_ip = iphdr->ip_hl << 2;
@@ -737,17 +740,17 @@ got_packet(u_char *args, const struct pcap_pkthdr *header, const u_char *packet)
 
 
     flags = tcphdr->th_flags;
-    /* Check correct SYN-ACK expecation, if so then proceed in fixing entire schedule from relative to absolute SEQs+ACKs */
+    /* Check correct SYN-ACK expectation, if so then proceed in fixing entire schedule from relative to absolute SEQs+ACKs */
     if((flags == (TH_SYN|TH_ACK)) && (sched_index==1) && (tcphdr->th_ack==htonl(sched[sched_index-1].curr_lseq + 1))){
         unsigned int j;
-        printf("Received Remote Packet...............	[%d]\n",sched_index+1);
+        printf("Received Remote Packet...............	[%u]\n", sched_index + 1);
         printf("Remote Pakcet Expectation met.\nProceeding in replay....\n");
         //printf("SYN-ACKed Random SEQ set!\n");
         initial_rseq = ntohl(tcphdr->th_seq); 
         //printf("initial_rseq: %u\n", initial_rseq);
         /* After we receiving the first SYN-ACK, then adjust the entire sched to be absolute rather than relative #s*/
         sched[1].exp_rseq = sched[1].exp_rseq + initial_rseq;
-        for(j = 2; j<pkts_scheduled; j++){ /* Based on correctly recieving the random SEQ from the SYN-ACK packet, do the following:*/
+        for(j = 2; j<pkts_scheduled; j++){ /* Based on correctly receiving the random SEQ from the SYN-ACK packet, do the following:*/
             if(sched[j].local){ /* Set local ACKs for entire sched to be absolute #s*/
                 sched[j].curr_lack = sched[j].curr_lack + initial_rseq; 
             }
@@ -936,21 +939,14 @@ int
 rewrite(input_addr* new_remoteip, struct mac_addr* new_remotemac, input_addr* myip, struct mac_addr* mymac, char* file, unsigned int new_src_port)
 {
 
-    ether_hdr* etherhdr; 
-    ipv4_hdr *iphdr;
-    tcp_hdr *tcphdr;
+    char *newfile = "newfile.pcap";
+    int pkt_counter;
     input_addr local_ip; 
     input_addr remote_ip; 
-    unsigned int size_ip;
-    unsigned int size_tcp; 
-    char* newfile = "newfile.pcap";
-    char ErrBuff [1024];
-    int pkt_counter;
     const u_char *packet;
     struct pcap_pkthdr *header;
     pcap_dumper_t *dumpfile;
     input_addr sip;  /* Source IP */ 
-    unsigned int flags;
     int local_packets = 0;
     bool initstep1 = false;  /* keep track of successful handshake step */
     bool warned = false;
@@ -967,6 +963,7 @@ rewrite(input_addr* new_remoteip, struct mac_addr* new_remotemac, input_addr* my
 
     pcap_t *pcap = set_offline_filter(file); 
     if (!pcap){
+        char ErrBuff [1024];
         fprintf (stderr, "Cannot open PCAP file '%s' for reading\n", file);
         fprintf(stderr, "%s\n",ErrBuff);
         return PCAP_OPEN_ERROR;
@@ -979,7 +976,12 @@ rewrite(input_addr* new_remoteip, struct mac_addr* new_remotemac, input_addr* my
     }
 
     /*Modify each packet's IP & MAC based on the passed args then do a checksum of each packet*/
-    for (pkt_counter = 0; pcap_next_ex(pcap, &header, &packet) > 0; pkt_counter++){
+    for (pkt_counter = 0; safe_pcap_next_ex(pcap, &header, &packet) > 0; pkt_counter++){
+        unsigned int flags, size_ip;
+        ether_hdr* etherhdr;
+        ipv4_hdr *iphdr;
+        tcp_hdr *tcphdr;
+        unsigned int size_tcp;
 
         if (!warned && header->len > header->caplen) {
             fprintf(stderr, "warning: packet capture truncated to %d byte packets\n",
@@ -1141,14 +1143,14 @@ fix_all_checksum_liveplay(ipv4_hdr *iphdr){
     /*Calculate TCP Checksum*/
     ret2 = do_checksum_liveplay((u_char *) iphdr,iphdr->ip_p, ntohs(iphdr->ip_len) - (iphdr->ip_hl << 2));
     if(ret2==-1){
-        printf("*******An Error Occured calculating TCP Checksum*******\n");
+        printf("*******An Error Occurred calculating TCP Checksum*******\n");
         return -1;
     }
 
     /*Calculate IP Checksum*/
     ret1 = do_checksum_liveplay((u_char *) iphdr, IPPROTO_IP, ntohs(iphdr->ip_len));
     if(ret1==-1){
-        printf("*******An Error Occured calculating IP Checksum*******\n");
+        printf("*******An Error Occurred calculating IP Checksum*******\n");
         return -1;
     }
 
@@ -1180,9 +1182,8 @@ do_checksum_liveplay(u_int8_t *data, int proto, int len) {
     ipv4_hdr *ipv4;
     tcp_hdr *tcp;
     int ip_hl;
-    volatile int sum;   // <-- volatile works around a PPC g++ bug
+    volatile int sum = 0;   // <-- volatile works around a PPC g++ bug
 
-    sum = 0;
     ipv4 = NULL;
 
     ipv4 = (ipv4_hdr *)data;
