@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2017 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2018 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -84,18 +84,14 @@ main(int argc, char *argv[])
     int out_file;
     COUNTER totpackets = 0;
     char errbuf[PCAP_ERRBUF_SIZE];
-    int optct = 0;
     tcpprep_opt_t *options;
  
     tcpprep = tcpprep_init();
     options = tcpprep->options;
     
-    optct = optionProcess(&tcpprepOptions, argc, argv);
+    optionProcess(&tcpprepOptions, argc, argv);
     tcpprep_post_args(tcpprep, argc, argv);
 
-    argc -= optct;
-    argv += optct;
-
     /* open the cache file */
     if ((out_file = open(OPT_ARG(CACHEFILE), O_WRONLY | O_CREAT | O_TRUNC,
             S_IREAD | S_IWRITE | S_IRGRP | S_IWGRP | S_IROTH)) == -1)
@@ -177,7 +173,7 @@ main(int argc, char *argv[])
         }
 
         if (info)
-            notice("Buliding cache file...\n");
+            notice("Building cache file...\n");
         /* 
          * re-process files, but this time generate
          * cache 
@@ -197,8 +193,9 @@ main(int argc, char *argv[])
 
     /* close cache file */
     close(out_file);
-    return 0;
 
+    restore_stdin();
+    return 0;
 }
 
 
@@ -294,7 +291,6 @@ check_ipv4_regex(const unsigned long ip)
     } else {
         return 0;
     }
-
 }
 
 static int
@@ -335,7 +331,7 @@ process_raw_packets(pcap_t * pcap)
 
     assert(pcap);
     
-    while ((pktdata = pcap_next(pcap, &pkthdr)) != NULL) {
+    while ((pktdata = safe_pcap_next(pcap, &pkthdr)) != NULL) {
         packetnum++;
 
         dbgx(1, "Packet " COUNTER_SPEC, packetnum);
@@ -343,11 +339,13 @@ process_raw_packets(pcap_t * pcap)
         /* look for include or exclude LIST match */
         if (options->xX.list != NULL) {
             if (options->xX.mode < xXExclude) {
+                /* include list */
                 if (!check_list(options->xX.list, packetnum)) {
                     add_cache(&(options->cachedata), DONT_SEND, 0);
                     continue;
                 }
             }
+            /* exclude list */
             else if (check_list(options->xX.list, packetnum)) {
                 add_cache(&(options->cachedata), DONT_SEND, 0);
                 continue;
@@ -359,9 +357,6 @@ process_raw_packets(pcap_t * pcap)
          * it as a non-IP packet, UNLESS we're in MAC mode, in which case
          * we should let the MAC matcher below handle it
          */
-
-        eth_hdr = (eth_hdr_t *)pktdata;
-
         if (options->mode != MAC_MODE) {
             dbg(3, "Looking for IPv4/v6 header in non-MAC mode");
             
@@ -372,17 +367,12 @@ process_raw_packets(pcap_t * pcap)
             if ((ip_hdr = (ipv4_hdr_t *)get_ipv4(pktdata, pkthdr.caplen, 
                     pcap_datalink(pcap), &buffptr))) {
                 dbg(2, "Packet is IPv4");
-                    
-            } 
-            
-            /* then look for IPv6 */
-            else if ((ip6_hdr = (ipv6_hdr_t *)get_ipv6(pktdata, pkthdr.caplen,
+            } else if ((ip6_hdr = (ipv6_hdr_t *)get_ipv6(pktdata, pkthdr.caplen,
                     pcap_datalink(pcap), &buffptr))) {
+                /* IPv6 */
                 dbg(2, "Packet is IPv6");    
-            } 
-            
-            /* we're something else... */
-            else {
+            } else {
+                /* we're something else... */
                 dbg(2, "Packet isn't IPv4/v6");
 
                 /* we don't want to cache these packets twice */
@@ -396,6 +386,10 @@ process_raw_packets(pcap_t * pcap)
             }
     
             l2len = get_l2len(pktdata, pkthdr.caplen, pcap_datalink(pcap));
+            if (l2len < 0) {
+                /* go to next packet */
+                continue;
+            }
 
             /* look for include or exclude CIDR match */
             if (options->xX.cidr != NULL) {
@@ -446,6 +440,12 @@ process_raw_packets(pcap_t * pcap)
 
         case MAC_MODE:
             dbg(2, "processing mac mode...");
+            if (pkthdr.caplen < sizeof(*eth_hdr)) {
+                dbg(2, "capture length too short for mac mode processing");
+                break;
+            }
+
+            eth_hdr = (eth_hdr_t *)pktdata;
             direction = macinstring(options->maclist, (u_char *)eth_hdr->ether_shost);
 
             /* reverse direction? */
@@ -460,15 +460,15 @@ process_raw_packets(pcap_t * pcap)
             /* first run through in auto mode: create tree */
             if (options->automode != FIRST_MODE) {
                 if (ip_hdr) {
-                    add_tree_ipv4(ip_hdr->ip_src.s_addr, pktdata);
+                    add_tree_ipv4(ip_hdr->ip_src.s_addr, pktdata, pkthdr.caplen);
                 } else if (ip6_hdr) {
-                    add_tree_ipv6(&ip6_hdr->ip_src, pktdata);
+                    add_tree_ipv6(&ip6_hdr->ip_src, pktdata, pkthdr.caplen);
                 }
             } else {
                 if (ip_hdr) {
-                    add_tree_first_ipv4(pktdata);
+                    add_tree_first_ipv4(pktdata, pkthdr.caplen);
                 } else if (ip6_hdr) {
-                    add_tree_first_ipv6(pktdata);
+                    add_tree_first_ipv6(pktdata, pkthdr.caplen);
                 }
             }  
             break;
@@ -539,7 +539,7 @@ process_raw_packets(pcap_t * pcap)
              */
             dbg(2, "processing port mode...");
             add_cache(&options->cachedata, SEND,
-            		check_dst_port(ip_hdr, ip6_hdr, (pkthdr.caplen - l2len)));
+            check_dst_port(ip_hdr, ip6_hdr, (pkthdr.caplen - l2len)));
             break;
 
         case FIRST_MODE:
@@ -598,6 +598,9 @@ print_info(const char *file)
     COUNTER count = 0, i;
 
     count = read_cache(&cachedata, file, &comment);
+    if (count > 65535)
+        exit(-1);
+
     for (i = 1; i <= count; i ++) {
         
         switch (check_cache(cachedata, i)) {
@@ -627,11 +630,11 @@ print_stats(const char *file)
 {
     char *cachedata = NULL;
     char *comment = NULL;
-    COUNTER count = 0;
+    COUNTER i, count = 0;
     COUNTER pri = 0, sec = 0, nosend = 0;
     
     count = read_cache(&cachedata, file, &comment);
-    for (COUNTER i = 1; i <= count; i ++) {
+    for (i = 1; i <= count; i ++) {
         int cacheval = check_cache(cachedata, i);
         switch (cacheval) {
             case TCPR_DIR_C2S:
