@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2017 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2018 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -58,7 +58,6 @@ tcpedit_t *tcpedit;
 /* local functions */
 void tcprewrite_init(void);
 void post_args(int argc, char *argv[]);
-void verify_input_pcap(pcap_t *pcap);
 int rewrite_packets(tcpedit_t *tcpedit, pcap_t *pin, pcap_dumper_t *pout);
 
 int 
@@ -98,7 +97,7 @@ main(int argc, char *argv[])
     }
 
     /* fuzzing init */
-    fuzzing_init(tcpedit->fuzz_seed);
+    fuzzing_init(tcpedit->fuzz_seed, tcpedit->fuzz_factor);
 
    /* open up the output file */
     options.outfile = safe_strdup(OPT_ARG(OUTFILE));
@@ -149,6 +148,8 @@ main(int argc, char *argv[])
 #ifdef ENABLE_DMALLOC
     dmalloc_shutdown();
 #endif
+
+    restore_stdin();
     return 0;
 }
 
@@ -256,7 +257,7 @@ rewrite_packets(tcpedit_t *tcpedit, pcap_t *pin, pcap_dumper_t *pout)
      * Keep sending while we have packets or until
      * we've sent enough packets
      */
-    while ((pktconst = pcap_next(pin, pkthdr_ptr)) != NULL) {
+    while ((pktconst = safe_pcap_next(pin, pkthdr_ptr)) != NULL) {
         packetnum++;
         dbgx(2, "packet " COUNTER_SPEC " caplen %d", packetnum, pkthdr.caplen);
 
